#pragma once

#include "FMM.h"
#include <map>
#include <vector>



class 	AugmentedFastMarchingMethod : public FastMarchingMethod
{
public:	

enum METHOD									//Method for computing the feature transform:
	{										//
	  AFMM_STAR,							// - one-point feature transform
	  AFMM_VDT								// - all-points feature transform (more expensive)
	};  

		AugmentedFastMarchingMethod(Image<float>* dt,FLAGS* shape,Image<float>* ft,Image<std::multimap<float,int> >* =0);
											//Ctor
											
int		execute(float max_distance=FLAGS::MYINFINITY,int max_iterations=MAX_ITERATIONS);		

void    setScanDir(int);					//Set scanning direction. arg can be 0 or 1, thus generating
											//hopefully totally different 'connection-lines'
void    setMethod(METHOD);					//Set way in which the count-field is computed
float	getLength();						//Get boundary-length computed in execute()
void    setDistTolerance(float);			//Set tolerance for comparing distances when computing VDT-origins
float   getDistTolerance();
								

protected:

void	add_to_narrowband(int,int,int,int);	//Enhance inherited to compute the full-feature-transform
void	init_count();						//Initializes 'count' from initial shape boundary		
int		diffuse();

Image<float>*		 count;					//Maintains the propagated boundary-parametrization
Image<int>*			 icount;				//Maintains the propagation of 'from'
std::vector<Coord>   from;					//Given a boundary-label, tells where, on initial boundary, it came from
Image<std::multimap<float,int> >* origins;	//Origin set
int					 scan_dir;				//Used in init_count()	
METHOD				 comp_method;			//Skeletonization method
float				 length;				//Boundary-length, used for wraparound-distance computations
float				 dist_tol;				//Tolerance under which two distances are equal. Influences #origins computed by VDT

};	




inline void AugmentedFastMarchingMethod::setScanDir(int d)
{  scan_dir = d;  }

inline void AugmentedFastMarchingMethod::setMethod(METHOD m)
{  comp_method = m;  }
   
inline float AugmentedFastMarchingMethod::getLength()
{  return length;  }

inline void AugmentedFastMarchingMethod::setDistTolerance(float dt)
{  dist_tol = dt;  }

inline float AugmentedFastMarchingMethod::getDistTolerance()
{  return dist_tol;  }


